@@ -1,5 +1,5 @@
 /*  Lziprecover - Data recovery tool for the lzip format
-    Copyright (C) 2009-2016 Antonio Diaz Diaz.
+    Copyright (C) 2009-2018 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -54,9 +54,9 @@ public:
 // defined in range_dec.cc
 int range_decompress( const std::string & input_filename,
                       const std::string & default_output_filename,
-                      Block range, const int verbosity, const bool force,
-                      const bool ignore, const bool to_stdout );
+                      Block range, const bool force, const bool ignore_errors,
+                      const bool ignore_trailing, const bool loose_trailing,
+                      const bool to_stdout );
 
 // defined in repair.cc
-int debug_delay( const std::string & input_filename, Block range,
-                 const int verbosity );
+int debug_delay( const std::string & input_filename, Block range );
