@@ -1,5 +1,5 @@
 /*  Lziprecover - Data recovery tool for the lzip format
-    Copyright (C) 2009-2015 Antonio Diaz Diaz.
+    Copyright (C) 2009-2018 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,10 +20,11 @@ class File_index
   struct Member
     {
     Block dblock, mblock;		// data block, member block
+    unsigned dictionary_size;
 
     Member( const long long dp, const long long ds,
-            const long long mp, const long long ms )
-      : dblock( dp, ds ), mblock( mp, ms ) {}
+            const long long mp, const long long ms, const unsigned dict_size )
+      : dblock( dp, ds ), mblock( mp, ms ), dictionary_size( dict_size ) {}
 
     bool operator==( const Member & m ) const { return ( mblock == m.mblock ); }
     bool operator!=( const Member & m ) const { return ( mblock != m.mblock ); }
@@ -35,12 +36,15 @@ class File_index
   int retval_;
 
   void set_errno_error( const char * const msg );
-  void set_num_error( const char * const msg1, unsigned long long num,
-                      const char * const msg2 = "." );
+  void set_num_error( const char * const msg, unsigned long long num );
+  bool skip_trailing_data( const int fd, long long & pos,
+         const bool ignore_bad_ds,
+         const bool ignore_trailing, const bool loose_trailing );
 
 public:
-  File_index() : error_( "No index." ), isize( 0 ), retval_( 2 ) {}
-  explicit File_index( const int infd );
+  File_index() : error_( "No index" ), isize( 0 ), retval_( 2 ) {}
+  File_index( const int infd, const bool ignore_bad_ds,
+              const bool ignore_trailing, const bool loose_trailing );
   File_index( const std::vector< int > & infd_vector, const long long fsize );
 
   long members() const { return member_vector.size(); }
@@ -57,15 +61,15 @@ public:
     }
   bool operator!=( const File_index & fi ) const { return !( *this == fi ); }
 
-  long long data_end() const
-    { if( member_vector.size() ) return member_vector.back().dblock.end();
-      else return 0; }
+  long long udata_size() const
+    { if( member_vector.empty() ) return 0;
+      return member_vector.back().dblock.end(); }
 
-  long long file_end() const
-    { if( member_vector.size() ) return member_vector.back().mblock.end();
-      else return 0; }
+  long long cdata_size() const
+    { if( member_vector.empty() ) return 0;
+      return member_vector.back().mblock.end(); }
 
-  // total size including trailing garbage (if any)
+  // total size including trailing data (if any)
   long long file_size() const
     { if( isize >= 0 ) return isize; else return 0; }
 
@@ -73,4 +77,6 @@ public:
     { return member_vector[i].dblock; }
   const Block & mblock( const long i ) const
     { return member_vector[i].mblock; }
+  unsigned dictionary_size( const long i ) const
+    { return member_vector[i].dictionary_size; }
   };
