@@ -1,5 +1,5 @@
 /*  Lziprecover - Data recovery tool for the lzip format
-    Copyright (C) 2009-2015 Antonio Diaz Diaz.
+    Copyright (C) 2009-2018 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 
 #define _FILE_OFFSET_BITS 64
 
+#include <algorithm>
 #include <cerrno>
 #include <climits>
 #include <cstdio>
@@ -36,12 +37,10 @@ namespace {
 
 void first_filename( const std::string & input_filename,
                      const std::string & default_output_filename,
-                     std::string & output_filename, const int max_digits )
+                     const int max_digits )
   {
-  if( default_output_filename.size() )
-    output_filename = default_output_filename;
-  else
-    output_filename = input_filename;
+  output_filename = default_output_filename.empty() ?
+                    input_filename : default_output_filename;
   int b = output_filename.size();
   while( b > 0 && output_filename[b-1] != '/' ) --b;
   output_filename.insert( b, "rec1" );
@@ -49,7 +48,7 @@ void first_filename( const std::string & input_filename,
   }
 
 
-bool next_filename( std::string & output_filename, const int max_digits )
+bool next_filename( const int max_digits )
   {
   int b = output_filename.size();
   while( b > 0 && output_filename[b-1] != '/' ) --b;
@@ -62,25 +61,6 @@ bool next_filename( std::string & output_filename, const int max_digits )
   }
 
 
-bool verify_header( const File_header & header, const Pretty_print & pp )
-  {
-  if( !header.verify_magic() )
-    {
-    pp( "Bad magic number (file not in lzip format)." );
-    return false;
-    }
-  if( !header.verify_version() )
-    {
-    if( pp.verbosity() >= 0 )
-      { pp();
-        std::fprintf( stderr, "Version %d member format not supported.\n",
-                      header.version() ); }
-    return false;
-    }
-  return true;
-  }
-
-
 // Search forward from 'pos' for "LZIP" (Boyer-Moore algorithm)
 // Returns pos of found string or 'pos+size' if not found.
 //
@@ -106,7 +86,7 @@ int find_magic( const uint8_t * const buffer, const int pos, const int size )
 
 int do_split_file( const std::string & input_filename, uint8_t * & base_buffer,
                    const std::string & default_output_filename,
-                   const int verbosity, const bool force )
+                   const bool force )
   {
   const int hsize = File_header::size;
   const int tsize = File_trailer::size;
@@ -118,7 +98,15 @@ int do_split_file( const std::string & input_filename, uint8_t * & base_buffer,
   struct stat in_stats;
   const int infd = open_instream( input_filename.c_str(), &in_stats, true, true );
   if( infd < 0 ) return 1;
-  Pretty_print pp( input_filename, verbosity );
+  Pretty_print pp( input_filename );
+
+  // don't move this after seek_read
+  const File_index file_index( infd, true, true, true );
+//  if( file_index.retval() != 0 ) pp( file_index.error().c_str() );
+  const long max_members = file_index.retval() ? 999999 : file_index.members();
+  int max_digits = 1;
+  for( long i = max_members; i >= 10; i /= 10 ) ++max_digits;
+
   int size = seek_read( infd, buffer, buffer_size + hsize, 0 ) - hsize;
   bool at_stream_end = ( size < buffer_size );
   if( size != buffer_size && errno )
@@ -127,19 +115,12 @@ int do_split_file( const std::string & input_filename, uint8_t * & base_buffer,
     { pp( "Input file is too short." ); return 2; }
   if( !verify_header( *(File_header *)buffer, pp ) ) return 2;
 
-  const File_index file_index( infd );
-  if( file_index.retval() != 0 ) pp( file_index.error().c_str() );
-  const long max_members = file_index.retval() ? 999999 : file_index.members();
-  int max_digits = 1;
-  for( long i = max_members; i >= 10; i /= 10 ) ++max_digits;
-
-  std::string output_filename;
-  first_filename( input_filename, default_output_filename, output_filename,
-                  max_digits );
-  int outfd = open_outstream_rw( output_filename, force );
-  if( outfd < 0 ) { close( infd ); return 1; }
+  first_filename( input_filename, default_output_filename, max_digits );
+  if( !open_outstream( force, false, false, false ) )
+    { close( infd ); return 1; }
 
   unsigned long long partial_member_size = 0;
+  const bool ttyout = isatty( STDOUT_FILENO );
   while( true )
     {
     int pos = 0;
@@ -154,17 +135,17 @@ int do_split_file( const std::string & input_filename, uint8_t * & base_buffer,
           const int wr = writeblock( outfd, buffer + pos, newpos - pos );
           if( wr != newpos - pos )
             { show_error( "Write error", errno ); return 1; }
-          if( close( outfd ) != 0 )
-            { show_error( "Error closing output file", errno ); return 1; }
+          if( close_outstream( &in_stats ) != 0 ) return 1;
           if( verbosity >= 1 )
             {
-            std::printf( "Member '%s' done \r", output_filename.c_str() );
+            std::printf( "Member '%s' done %c", output_filename.c_str(),
+                         ttyout ? '\r' : '\n' );
             std::fflush( stdout );
             }
-          if( !next_filename( output_filename, max_digits ) )
+          if( !next_filename( max_digits ) )
             { show_error( "Too many members in file." ); close( infd ); return 1; }
-          outfd = open_outstream_rw( output_filename, force );
-          if( outfd < 0 ) { close( infd ); return 1; }
+          if( !open_outstream( force, false, false, false ) )
+            { close( infd ); return 1; }
           partial_member_size = 0;
           pos = newpos;
           }
@@ -192,8 +173,7 @@ int do_split_file( const std::string & input_filename, uint8_t * & base_buffer,
       { show_error( "Read error", errno ); return 1; }
     }
   close( infd );
-  if( close( outfd ) != 0 )
-    { show_error( "Error closing output file", errno ); return 1; }
+  if( close_outstream( &in_stats ) != 0 ) return 1;
   if( verbosity >= 1 )
     {
     std::printf( "Member '%s' done \n", output_filename.c_str() );
@@ -205,13 +185,22 @@ int do_split_file( const std::string & input_filename, uint8_t * & base_buffer,
 } // end namespace
 
 
+bool verify_header( const File_header & header, const Pretty_print & pp )
+  {
+  if( !header.verify_magic() )
+    { pp( bad_magic_msg ); return false; }
+  if( !header.verify_version() )
+    { pp( bad_version( header.version() ) ); return false; }
+  return true;
+  }
+
+
 int split_file( const std::string & input_filename,
-                const std::string & default_output_filename,
-                const int verbosity, const bool force )
+                const std::string & default_output_filename, const bool force )
   {
   uint8_t * base_buffer;
   const int retval = do_split_file( input_filename, base_buffer,
-                                    default_output_filename, verbosity, force );
+                                    default_output_filename, force );
   delete[] base_buffer;
   return retval;
   }
